@@ -620,147 +620,147 @@ set_key_label_in_layout (GkbdKeyboardDrawingRenderContext * context,
 	PangoLayout *layout = context->layout;
 
 	switch (keyval) {
-	case GDK_Scroll_Lock:
+	case GDK_KEY_Scroll_Lock:
 		pango_layout_set_text (layout, "Scroll\nLock", -1);
 		break;
 
-	case GDK_space:
+	case GDK_KEY_space:
 		pango_layout_set_text (layout, "", -1);
 		break;
 
-	case GDK_Sys_Req:
+	case GDK_KEY_Sys_Req:
 		pango_layout_set_text (layout, "Sys Rq", -1);
 		break;
 
-	case GDK_Page_Up:
+	case GDK_KEY_Page_Up:
 		pango_layout_set_text (layout, "Page\nUp", -1);
 		break;
 
-	case GDK_Page_Down:
+	case GDK_KEY_Page_Down:
 		pango_layout_set_text (layout, "Page\nDown", -1);
 		break;
 
-	case GDK_Num_Lock:
+	case GDK_KEY_Num_Lock:
 		pango_layout_set_text (layout, "Num\nLock", -1);
 		break;
 
-	case GDK_KP_Page_Up:
+	case GDK_KEY_KP_Page_Up:
 		pango_layout_set_text (layout, "Pg Up", -1);
 		break;
 
-	case GDK_KP_Page_Down:
+	case GDK_KEY_KP_Page_Down:
 		pango_layout_set_text (layout, "Pg Dn", -1);
 		break;
 
-	case GDK_KP_Home:
+	case GDK_KEY_KP_Home:
 		pango_layout_set_text (layout, "Home", -1);
 		break;
 
-	case GDK_KP_Left:
+	case GDK_KEY_KP_Left:
 		pango_layout_set_text (layout, "Left", -1);
 		break;
 
-	case GDK_KP_End:
+	case GDK_KEY_KP_End:
 		pango_layout_set_text (layout, "End", -1);
 		break;
 
-	case GDK_KP_Up:
+	case GDK_KEY_KP_Up:
 		pango_layout_set_text (layout, "Up", -1);
 		break;
 
-	case GDK_KP_Begin:
+	case GDK_KEY_KP_Begin:
 		pango_layout_set_text (layout, "Begin", -1);
 		break;
 
-	case GDK_KP_Right:
+	case GDK_KEY_KP_Right:
 		pango_layout_set_text (layout, "Right", -1);
 		break;
 
-	case GDK_KP_Enter:
+	case GDK_KEY_KP_Enter:
 		pango_layout_set_text (layout, "Enter", -1);
 		break;
 
-	case GDK_KP_Down:
+	case GDK_KEY_KP_Down:
 		pango_layout_set_text (layout, "Down", -1);
 		break;
 
-	case GDK_KP_Insert:
+	case GDK_KEY_KP_Insert:
 		pango_layout_set_text (layout, "Ins", -1);
 		break;
 
-	case GDK_KP_Delete:
+	case GDK_KEY_KP_Delete:
 		pango_layout_set_text (layout, "Del", -1);
 		break;
 
-	case GDK_dead_grave:
+	case GDK_KEY_dead_grave:
 		pango_layout_set_text (layout, "ˋ", -1);
 		break;
 
-	case GDK_dead_acute:
+	case GDK_KEY_dead_acute:
 		pango_layout_set_text (layout, "ˊ", -1);
 		break;
 
-	case GDK_dead_circumflex:
+	case GDK_KEY_dead_circumflex:
 		pango_layout_set_text (layout, "ˆ", -1);
 		break;
 
-	case GDK_dead_tilde:
+	case GDK_KEY_dead_tilde:
 		pango_layout_set_text (layout, "~", -1);
 		break;
 
-	case GDK_dead_macron:
+	case GDK_KEY_dead_macron:
 		pango_layout_set_text (layout, "ˉ", -1);
 		break;
 
-	case GDK_dead_breve:
+	case GDK_KEY_dead_breve:
 		pango_layout_set_text (layout, "˘", -1);
 		break;
 
-	case GDK_dead_abovedot:
+	case GDK_KEY_dead_abovedot:
 		pango_layout_set_text (layout, "˙", -1);
 		break;
 
-	case GDK_dead_diaeresis:
+	case GDK_KEY_dead_diaeresis:
 		pango_layout_set_text (layout, "¨", -1);
 		break;
 
-	case GDK_dead_abovering:
+	case GDK_KEY_dead_abovering:
 		pango_layout_set_text (layout, "˚", -1);
 		break;
 
-	case GDK_dead_doubleacute:
+	case GDK_KEY_dead_doubleacute:
 		pango_layout_set_text (layout, "˝", -1);
 		break;
 
-	case GDK_dead_caron:
+	case GDK_KEY_dead_caron:
 		pango_layout_set_text (layout, "ˇ", -1);
 		break;
 
-	case GDK_dead_cedilla:
+	case GDK_KEY_dead_cedilla:
 		pango_layout_set_text (layout, "¸", -1);
 		break;
 
-	case GDK_dead_ogonek:
+	case GDK_KEY_dead_ogonek:
 		pango_layout_set_text (layout, "˛", -1);
 		break;
 
-		/* case GDK_dead_iota:
-		 * case GDK_dead_voiced_sound:
-		 * case GDK_dead_semivoiced_sound: */
+		/* case GDK_KEY_dead_iota:
+		 * case GDK_KEY_dead_voiced_sound:
+		 * case GDK_KEY_dead_semivoiced_sound: */
 
-	case GDK_dead_belowdot:
+	case GDK_KEY_dead_belowdot:
 		pango_layout_set_text (layout, " ̣", -1);
 		break;
 
-	case GDK_horizconnector:
+	case GDK_KEY_horizconnector:
 		pango_layout_set_text (layout, "horiz\nconn", -1);
 		break;
 
-	case GDK_Mode_switch:
+	case GDK_KEY_Mode_switch:
 		pango_layout_set_text (layout, "AltGr", -1);
 		break;
 
-	case GDK_Multi_key:
+	case GDK_KEY_Multi_key:
 		pango_layout_set_text (layout, "Compose", -1);
 		break;
 
@@ -1101,8 +1101,9 @@ invalidate_indicator_doodad_region (GkbdKeyboardDrawing * drawing,
 			   doodad->doodad->indicator.left,
 			   doodad->origin_y +
 			   doodad->doodad->indicator.top,
-			   &drawing->xkb->geom->shapes[doodad->
-						       doodad->indicator.shape_ndx]);
+			   &drawing->xkb->geom->shapes[doodad->doodad->
+						       indicator.
+						       shape_ndx]);
 }
 
 static void
@@ -1116,8 +1117,8 @@ invalidate_key_region (GkbdKeyboardDrawing * drawing,
 			   key->angle,
 			   key->origin_x,
 			   key->origin_y,
-			   &drawing->xkb->geom->shapes[key->
-						       xkbkey->shape_ndx]);
+			   &drawing->xkb->geom->shapes[key->xkbkey->
+						       shape_ndx]);
 }
 
 static void
@@ -1301,10 +1302,10 @@ static gboolean
 create_cairo (GkbdKeyboardDrawing * drawing)
 {
 	GtkStateType state;
-	if (drawing == NULL || drawing->pixmap == NULL)
+	if (drawing == NULL || drawing->surface == NULL)
 		return FALSE;
 	drawing->renderContext->cr =
-	    gdk_cairo_create (GDK_DRAWABLE (drawing->pixmap));
+	    gdk_cairo_create (GDK_DRAWABLE (drawing->surface));
 
 	state = gtk_widget_get_state (GTK_WIDGET (drawing));
 	drawing->renderContext->dark_color =
@@ -1324,7 +1325,7 @@ static void
 draw_keyboard (GkbdKeyboardDrawing * drawing)
 {
 	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
-        GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (drawing));
+	GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (drawing));
 	GtkAllocation allocation;
 
 	if (!drawing->xkb)
@@ -1332,15 +1333,15 @@ draw_keyboard (GkbdKeyboardDrawing * drawing)
 
 	gtk_widget_get_allocation (GTK_WIDGET (drawing), &allocation);
 
-	drawing->pixmap =
-	    gdk_pixmap_new (gtk_widget_get_window (GTK_WIDGET (drawing)),
+	drawing->surface =
+	    gdk_window_create_similar_surface (gtk_widget_get_window (GTK_WIDGET (drawing)),
 			    allocation.width, allocation.height, -1);
 
 	if (create_cairo (drawing)) {
-	        /* blank background */
-                gdk_cairo_set_source_color (drawing->renderContext->cr,
-                                            &style->base[state]);
-                cairo_paint (drawing->renderContext->cr);
+		/* blank background */
+		gdk_cairo_set_source_color (drawing->renderContext->cr,
+					    &style->base[state]);
+		cairo_paint (drawing->renderContext->cr);
 
 		draw_keyboard_to_context (drawing->renderContext, drawing);
 		destroy_cairo (drawing);
@@ -1380,33 +1381,26 @@ free_render_context (GkbdKeyboardDrawing * drawing)
 }
 
 static gboolean
-expose_event (GtkWidget * widget,
-	      GdkEventExpose * event, GkbdKeyboardDrawing * drawing)
+draw (GtkWidget * widget,
+      cairo_t *cr, GkbdKeyboardDrawing * drawing)
 {
 	GtkAllocation allocation;
-        cairo_t *cr;
 
 	if (!drawing->xkb)
 		return FALSE;
 
-	if (drawing->pixmap == NULL)
+	if (drawing->surface == NULL)
 		return FALSE;
 
-        cr = gdk_cairo_create (event->window);
-        gdk_cairo_region (cr, event->region);
-        cairo_clip (cr);
-
-        gdk_cairo_set_source_pixmap (cr, drawing->pixmap, 0, 0);
+        cairo_set_source_surface (cr, drawing->surface, 0, 0);
         cairo_paint (cr);
 
-        cairo_destroy (cr);
-
 	if (gtk_widget_has_focus (widget)) {
 		gtk_widget_get_allocation (widget, &allocation);
 		gtk_paint_focus (gtk_widget_get_style (widget),
-				 gtk_widget_get_window (widget),
+				 cr,
 				 gtk_widget_get_state (widget),
-				 &event->area, widget, "keyboard-drawing",
+				 widget, "keyboard-drawing",
 				 0, 0, allocation.width,
 				 allocation.height);
 	}
@@ -1469,9 +1463,9 @@ size_allocate (GtkWidget * widget,
 {
 	GkbdKeyboardDrawingRenderContext *context = drawing->renderContext;
 
-	if (drawing->pixmap) {
-		g_object_unref (drawing->pixmap);
-		drawing->pixmap = NULL;
+	if (drawing->surface) {
+                cairo_surface_destroy (drawing->surface);
+		drawing->surface = NULL;
 	}
 
 	if (!context_setup_scaling (context, drawing,
@@ -1686,8 +1680,8 @@ init_keys_and_doodads (GkbdKeyboardDrawing * drawing)
 				    drawing->xkb->geom->shapes +
 				    xkbkey->shape_ndx;
 				guint keycode = find_keycode (drawing,
-							      xkbkey->
-							      name.name);
+							      xkbkey->name.
+							      name);
 
 				if (keycode == INVALID_KEYCODE)
 					continue;
@@ -1798,9 +1792,8 @@ init_colors (GkbdKeyboardDrawing * drawing)
 
 	for (i = 0; i < drawing->xkb->geom->num_colors; i++) {
 		result =
-		    parse_xkb_color_spec (drawing->xkb->geom->
-					  colors[i].spec,
-					  drawing->colors + i);
+		    parse_xkb_color_spec (drawing->xkb->geom->colors[i].
+					  spec, drawing->colors + i);
 
 		if (!result)
 			g_warning
@@ -1930,8 +1923,8 @@ xkb_state_notify_event_filter (GdkXEvent * gdkxev,
 				process_indicators_state_notify (&
 								 ((XkbEvent
 								   *)
-								  gdkxev)->indicators,
-drawing);
+								  gdkxev)->
+indicators, drawing);
 			}
 			break;
 
@@ -1973,7 +1966,9 @@ destroy (GkbdKeyboardDrawing * drawing)
 		drawing->idle_redraw = 0;
 	}
 
-	g_object_unref (drawing->pixmap);
+        if (drawing->surface != NULL) {
+                cairo_surface_destroy (drawing->surface);
+        }
 }
 
 static void
@@ -2010,7 +2005,7 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 		drawing->screen_num =
 		    gdk_screen_get_number (gdk_screen_get_default ());
 
-	drawing->pixmap = NULL;
+	drawing->surface = NULL;
 	alloc_render_context (drawing);
 
 	drawing->keyboard_items = NULL;
@@ -2037,7 +2032,7 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 
 	XkbGetNames (drawing->display, XkbAllNamesMask, drawing->xkb);
 	drawing->l3mod = XkbKeysymToModifiers (drawing->display,
-					       GDK_ISO_Level3_Shift);
+					       GDK_KEY_ISO_Level3_Shift);
 
 	drawing->xkbOnDisplay = TRUE;
 
@@ -2071,8 +2066,8 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 			       GDK_EXPOSURE_MASK | GDK_KEY_PRESS_MASK |
 			       GDK_KEY_RELEASE_MASK | GDK_BUTTON_PRESS_MASK
 			       | GDK_FOCUS_CHANGE_MASK);
-	g_signal_connect (G_OBJECT (drawing), "expose-event",
-			  G_CALLBACK (expose_event), drawing);
+	g_signal_connect (G_OBJECT (drawing), "draw",
+			  G_CALLBACK (draw), drawing);
 	g_signal_connect_after (G_OBJECT (drawing), "key-press-event",
 				G_CALLBACK (key_event), drawing);
 	g_signal_connect_after (G_OBJECT (drawing), "key-release-event",
@@ -2156,24 +2151,6 @@ gkbd_keyboard_drawing_set_mods (GkbdKeyboardDrawing * drawing, guint mods)
 	}
 }
 
-/* returns a pixbuf with the keyboard drawing at the current pixel size
- * (which can then be saved to disk, etc) */
-GdkPixbuf *
-gkbd_keyboard_drawing_get_pixbuf (GkbdKeyboardDrawing * drawing)
-{
-	GkbdKeyboardDrawingRenderContext *context = drawing->renderContext;
-
-	if (drawing->pixmap == NULL)
-		draw_keyboard (drawing);
-
-	return gdk_pixbuf_get_from_drawable (NULL, drawing->pixmap, NULL,
-					     0, 0, 0, 0,
-					     xkb_to_pixmap_coord (context,
-								  drawing->xkb->geom->width_mm),
-					     xkb_to_pixmap_coord (context,
-								  drawing->xkb->geom->height_mm));
-}
-
 /**
  * gkbd_keyboard_drawing_render:
  * @drawing: keyboard layout to render
@@ -2524,7 +2501,9 @@ gkbd_keyboard_drawing_new_dialog (gint group, gchar * group_name)
 	GdkRectangle *rect;
 	GError *error = NULL;
 	char title[128] = "";
-	XklEngine *engine = xkl_engine_get_instance (GDK_DISPLAY ());
+	XklEngine *engine =
+	    xkl_engine_get_instance (GDK_DISPLAY_XDISPLAY
+				     (gdk_display_get_default ()));
 
 	builder = gtk_builder_new ();
 	gtk_builder_add_from_file (builder, UIDIR "/show-layout.ui",
@@ -2614,7 +2593,7 @@ gkbd_keyboard_drawing_new_dialog (gint group, gchar * group_name)
 
 	g_object_set_data (G_OBJECT (dialog), "kbdraw", kbdraw);
 
-	g_signal_connect_swapped (GTK_OBJECT (dialog), "destroy",
+	g_signal_connect_swapped (dialog, "destroy",
 				  G_CALLBACK (g_object_unref),
 				  g_object_get_data (G_OBJECT (dialog),
 						     "builderData"));
