@@ -123,7 +123,7 @@ struct _GkbdKeyboardDrawing {
 
 	GtkDrawingArea parent;
 
-	GdkPixmap *pixmap;
+	cairo_surface_t *surface;
 	XkbDescRec *xkb;
 	gboolean xkbOnDisplay;
 	guint l3mod;
@@ -209,7 +209,8 @@ void gkbd_keyboard_drawing_print (GkbdKeyboardDrawing * drawing,
 				  const gchar * description);
 
 
-GtkWidget* gkbd_keyboard_drawing_new_dialog (gint group, gchar* group_name);
+GtkWidget *gkbd_keyboard_drawing_new_dialog (gint group,
+					     gchar * group_name);
 
 G_END_DECLS
 #endif				/* #ifndef GKBD_KEYBOARD_DRAWING_H */
